<?php

namespace Drupal\computed_breadcrumbs\Field;

use Drupal\computed_breadcrumbs\Routing\BreadcrumbsResponse;
use Drupal\Core\Field\FieldItemList;
use Drupal\Core\TypedData\ComputedItemListTrait;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\HttpKernelInterface;

class ComputedBreadcrumbsItemList extends FieldItemList {

  use ComputedItemListTrait;

  /**
   * @inheritDoc
   */
  protected function computeValue() {
    /** @var \Symfony\Component\HttpKernel\HttpKernel $httpKernel */
    $httpKernel = \Drupal::service('http_kernel');
    /** @var \Symfony\Component\HttpFoundation\RequestStack $requestStack */
    $requestStack = \Drupal::service('request_stack');

    $parent = $this->getEntity();
    $url = $parent->toUrl();

    $currentRequest = $requestStack->getCurrentRequest();
    $request = Request::create(
      $url->getInternalPath(),
      'GET',
      $currentRequest->query->all(),
      $currentRequest->cookies->all(),
      $currentRequest->files->all(),
      $currentRequest->server->all()
    );

    if ($session = $currentRequest->getSession()) {
      $request->setSession($session);
    }

    $request->attributes->set('computed_breadcrumbs', TRUE);
    $response = $httpKernel->handle($request, HttpKernelInterface::SUB_REQUEST);

    // TODO:
    // Remove the request stack manipulation once the core issue described at
    // https://www.drupal.org/node/2613044 is resolved.
    while ($requestStack->getCurrentRequest() === $request) {
      $requestStack->pop();
    }

    if ($response instanceof BreadcrumbsResponse) {
      $links = $response->getBreadcrumbs();
      foreach ($links as $index => $link) {
        $this->list[] = $this->createItem($index, [
          'uri' => $link->getUrl()->setAbsolute()->toString(),
          'title' => $link->getText()
        ]);
      }
    }
  }

}
