<?php

namespace Drupal\KernelTests\computed_breadcrumbs\Kernel;

use Drupal\Core\Breadcrumb\Breadcrumb;
use Drupal\Core\Link;
use Drupal\Core\Routing\RouteMatch;
use Drupal\Core\Url;
use Drupal\KernelTests\Core\Entity\EntityKernelTestBase;
use Drupal\node\Entity\Node;
use Drupal\node\Entity\NodeType;
use Prophecy\Argument;

/**
 * Tests node validation constraints.
 *
 * @group node
 */
class ComputedBreadcrumbsKernelTest extends EntityKernelTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = ['node', 'computed_breadcrumbs'];

  /**
   * Set the default field storage backend for fields created during tests.
   */
  protected function setUp() {
    parent::setUp();

    $account = $this->createUser();
    $this->container->get('current_user')->setAccount($account);

    // Create a node type for testing.
    $type = NodeType::create(['type' => 'page', 'name' => 'page']);
    $type->save();

    $breadcrumbManager = $this->prophesize('Drupal\Core\Breadcrumb\BreadcrumbManager');

    $breadcrumbManager->build(Argument::any())
      ->will(function ($args) {
        $routeMatch = $args[0];
        $breadcrumb = new Breadcrumb();
        if ($routeMatch->getRouteName() == 'entity.node.canonical') {
          $breadcrumb->addLink(new Link('Test breadcrumb', Url::fromUserInput('/breadcrumbs-test')));
        }
        return $breadcrumb;
      });
    $this->container->set('breadcrumb', $breadcrumbManager->reveal());
  }

  /**
   * Tests the computed breadcrumbs field.
   */
  public function testComputedBreadcrumbs() {
    $node = Node::create(['type' => 'page', 'title' => 'test', 'uid' => 1]);
    $node->save();
    $breadcrumbs = $node->get('breadcrumbs')->getValue();
    $this->assertEquals(1, count($breadcrumbs));
    $this->assertEquals('Test breadcrumb', $breadcrumbs[0]['title']);
  }

}
